@@ -1,18 +1,34 @@
 import { PanelModel } from './PanelModel';
 import { getPanelPlugin } from '../../plugins/__mocks__/pluginMocks';
-import { ReactPanelPlugin } from '@grafana/ui/src/types/panel';
+
+class TablePanelCtrl {}
 
 describe('PanelModel', () => {
   describe('when creating new panel model', () => {
     let model;
     let modelJson;
+    let persistedOptionsMock;
+    const defaultOptionsMock = {
+      fieldOptions: {
+        thresholds: [
+          {
+            color: '#F2495C',
+            index: 1,
+            value: 50,
+          },
+          {
+            color: '#73BF69',
+            index: 0,
+            value: null,
+          },
+        ],
+      },
+      showThresholds: true,
+    };
 
     beforeEach(() => {
-      modelJson = {
-        type: 'table',
-        showColumns: true,
-        targets: [{ refId: 'A' }, { noRefId: true }],
-        options: {
+      persistedOptionsMock = {
+        fieldOptions: {
           thresholds: [
             {
               color: '#F2495C',
@@ -27,14 +43,34 @@ describe('PanelModel', () => {
           ],
         },
       };
+
+      modelJson = {
+        type: 'table',
+        showColumns: true,
+        targets: [{ refId: 'A' }, { noRefId: true }],
+        options: persistedOptionsMock,
+      };
+
       model = new PanelModel(modelJson);
-      model.pluginLoaded(getPanelPlugin({ id: 'table', exports: { PanelCtrl: {} as any } }));
+      const panelPlugin = getPanelPlugin(
+        {
+          id: 'table',
+        },
+        null, // react
+        TablePanelCtrl // angular
+      );
+      panelPlugin.setDefaults(defaultOptionsMock);
+      model.pluginLoaded(panelPlugin);
     });
 
     it('should apply defaults', () => {
       expect(model.gridPos.h).toBe(3);
     });
 
+    it('should apply option defaults', () => {
+      expect(model.getOptions().showThresholds).toBeTruthy();
+    });
+
     it('should set model props on instance', () => {
       expect(model.showColumns).toBe(true);
     });
@@ -63,7 +99,7 @@ describe('PanelModel', () => {
     });
 
     it('should restore -Infinity value for base threshold', () => {
-      expect(model.options.thresholds).toEqual([
+      expect(model.options.fieldOptions.thresholds).toEqual([
         {
           color: '#F2495C',
           index: 1,
@@ -78,24 +114,40 @@ describe('PanelModel', () => {
     });
 
     describe('when changing panel type', () => {
+      const newPanelPluginDefaults = {
+        showThresholdLabels: false,
+      };
+
       beforeEach(() => {
-        model.changePlugin(getPanelPlugin({ id: 'graph', exports: {} }));
+        const newPlugin = getPanelPlugin({ id: 'graph' });
+        newPlugin.setDefaults(newPanelPluginDefaults);
+        model.changePlugin(newPlugin);
         model.alert = { id: 2 };
       });
 
+      it('should apply next panel option defaults', () => {
+        expect(model.getOptions().showThresholdLabels).toBeFalsy();
+        expect(model.getOptions().showThresholds).toBeUndefined();
+      });
+
       it('should remove table properties but keep core props', () => {
         expect(model.showColumns).toBe(undefined);
       });
 
       it('should restore table properties when changing back', () => {
-        model.changePlugin(getPanelPlugin({ id: 'table', exports: {} }));
+        model.changePlugin(getPanelPlugin({ id: 'table' }));
         expect(model.showColumns).toBe(true);
       });
 
       it('should remove alert rule when changing type that does not support it', () => {
-        model.changePlugin(getPanelPlugin({ id: 'table', exports: {} }));
+        model.changePlugin(getPanelPlugin({ id: 'table' }));
         expect(model.alert).toBe(undefined);
       });
+
+      it('panelQueryRunner should be cleared', () => {
+        const panelQueryRunner = (model as any).queryRunner;
+        expect(panelQueryRunner).toBeFalsy();
+      });
     });
 
     describe('when changing from angular panel', () => {
@@ -105,7 +157,7 @@ describe('PanelModel', () => {
         model.events.on('panel-teardown', () => {
           tearDownPublished = true;
         });
-        model.changePlugin(getPanelPlugin({ id: 'graph', exports: {} }));
+        model.changePlugin(getPanelPlugin({ id: 'graph' }));
       });
 
       it('should teardown / destroy panel so angular panels event subscriptions are removed', () => {
@@ -114,39 +166,27 @@ describe('PanelModel', () => {
       });
     });
 
-    describe('when changing to react panel', () => {
+    describe('when changing to react panel from angular panel', () => {
+      let panelQueryRunner: any;
+
       const onPanelTypeChanged = jest.fn();
-      const reactPanel = new ReactPanelPlugin({} as any).setPanelChangeHandler(onPanelTypeChanged as any);
+      const reactPlugin = getPanelPlugin({ id: 'react' }).setPanelChangeHandler(onPanelTypeChanged as any);
 
       beforeEach(() => {
-        model.changePlugin(
-          getPanelPlugin({
-            id: 'react',
-            exports: {
-              reactPanel,
-            },
-          })
-        );
+        model.changePlugin(reactPlugin);
+        panelQueryRunner = model.getQueryRunner();
       });
 
       it('should call react onPanelTypeChanged', () => {
         expect(onPanelTypeChanged.mock.calls.length).toBe(1);
         expect(onPanelTypeChanged.mock.calls[0][1]).toBe('table');
-        expect(onPanelTypeChanged.mock.calls[0][2].thresholds).toBeDefined();
+        expect(onPanelTypeChanged.mock.calls[0][2].fieldOptions.thresholds).toBeDefined();
       });
-    });
 
-    describe('get panel options', () => {
-      it('should apply defaults', () => {
-        model.options = { existingProp: 10 };
-        const options = model.getOptions({
-          defaultProp: true,
-          existingProp: 0,
-        });
-
-        expect(options.defaultProp).toBe(true);
-        expect(options.existingProp).toBe(10);
-        expect(model.options).toBe(options);
+      it('getQueryRunner() should return same instance after changing to another react panel', () => {
+        model.changePlugin(getPanelPlugin({ id: 'react2' }));
+        const sameQueryRunner = model.getQueryRunner();
+        expect(panelQueryRunner).toBe(sameQueryRunner);
       });
     });
   });
